/* IAVLaserFeatureProxy.hpp
 *
 * Scott Livingston <SCL@utk.edu>
 * March 2009
 *
 * NOTES: - Based on pv3d, erf and miarn code by Joao Xavier <smogzer_at_gmail.com>.
 *        - See test_direct.cpp under examples directory for the basic
 *          process upon which this class attempts to build.
 *        - See test_class.cpp for a brief demo of this class IAVLaserFeatureProxy.
 */

#ifndef _IAVLaserFeatureProxy_HPP_
#define _IAVLaserFeatureProxy_HPP_


#include <libplayerc++/playerc++.h>

// miarn objects/routines
#include "fiducial.hpp"
#include "feature_geometry.hpp"
#include "feature_person.hpp"


class IAVLaserFeatureProxy : public PlayerCc::LaserProxy {

public:

	// constructor
	IAVLaserFeatureProxy( PlayerCc::PlayerClient *aPc, uint32_t aIndex = 0 );

	// Update feature/tracking data
    void Update();

	/* List numbers of basic features and locations of persons (i.e.,
	   pairs of legs) on cout stream. */
	void PrintFeatureCounts() const;

	// Access to extracted features.
	miarn_fiducial_item GetFeature( int index ) const;
	int GetNumFeatures() const
	{ return laserFeature.fdata.size(); }

	// Access to `leg pair' tracking (so-called `person' features).
	miarn_fiducial_item GetPersonFeature( int index ) const;
	int GetNumPersonFeatures() const
	{ return legTrack.fdata_out.size(); }
	
private:

	LaserFeatureX laserFeature;
	FeatureLegTracker legTrack;

};


#endif
