/* testcv.cc
 *
 * Use Player to read and display images from an attached camera until
 * the user presses ESC (escape).
 *
 * Scott Livingston
 * Originally created: 25 Jan 2009
 * Current: 4 Mar 2009
 */


#include <cv.h>
#include <highgui.h>

#include <libplayerc++/playerc++.h>
using namespace PlayerCc;

#include <iostream>
using namespace std;


int main( int argc, char *argv[] )
{
	uint cam_width, cam_height, cam_depth;
	uint8_t *imgBuf;
	IplImage *img;
	uint i, j; // generic counters

	PlayerClient robot( "localhost" );
	CameraProxy cp( &robot, 0 );

	// determine camera image specs
	robot.Read();
	cam_width = cp.GetWidth();
	cam_height = cp.GetHeight();
	cam_depth = cp.GetDepth();
	
	// Verbosity
	cout << "image size: " << cam_width << "x" << cam_height << " (" << cam_depth << " bpp)" << endl;

	// image buffer and OpenGL image structure
	imgBuf = new uint8_t[cam_width*cam_height*cam_depth];
	img = cvCreateImage( cvSize(cam_width, cam_height), IPL_DEPTH_8U, 3 );

	// open window for displaying camera image
	cvNamedWindow( "testWin", CV_WINDOW_AUTOSIZE );

	try {
		
		// loop until ESCAPE key is pressed
		while (cvWaitKey(33) != 27) {

			// capture most recent camera image
			robot.Read();
			cp.Decompress();
			cp.GetImage( imgBuf );

			// copy image from camera proxy
			for (i = 0; i < cam_width; i++) {
				for (j = 0; j < cam_height; j++) {
					img->imageData[cam_width * j*3 + i*3 + 0] = (char)imgBuf[cam_width * j*3 + i*3 + 2];
					img->imageData[cam_width * j*3 + i*3 + 1] = (char)imgBuf[cam_width * j*3 + i*3 + 1];
					img->imageData[cam_width * j*3 + i*3 + 2] = (char)imgBuf[cam_width * j*3 + i*3 + 0];
				}
			}

			cvShowImage( "testWin", img );

		}

	} catch (PlayerError e) {
		cerr << e << endl;
		return -1;
	}
	
	// release image buffer and image structure
	cvReleaseImage( &img );
	delete [] imgBuf;

	return 0;
}
