/* testKOT.cpp - Example using KernelObjTracker class.
 *
 * Scott Livingston <SCL@utk.edu>
 * Originally created: 18 April 2009
 *
 */


#include <cv.h>
#include <highgui.h>

#include <cstdlib>
#include <iostream>
using namespace std;

#include "KernelObjTracker.hpp"


int main( int argc, char **argv )
{
	IplImage *img,
		*dispImage; // Use bounding boxes for visual feedback
	CvCapture *capture;

	CvRect targetRegion;

	if (argc != 6) {
		cerr << "Usage: " << argv[0] << " init_image x0 y0 width height" << endl;
		return 1;
	}

	// Attempt to load target initialization image
	if ((img = cvLoadImage( argv[1] )) == NULL) {
		cerr << "Error: Could not open image file: " << argv[1] << endl;
		return -1;
	}

	// Open capture device, by default a webcam at /dev/video0
	capture = cvCaptureFromCAM( -1 );
	cvSetCaptureProperty( capture, CV_CAP_PROP_FRAME_WIDTH, 640 );
    cvSetCaptureProperty( capture, CV_CAP_PROP_FRAME_HEIGHT, 480 );
	if (!capture) {
		cerr << "Error: OpenCV capture object failed to open.\nCheck your webcam..." << endl;
		return -1;
	}

	// Verify sign of given region arguments
	if (atoi(argv[2]) < 0 || atoi(argv[3]) < 0
		|| atoi(argv[4]) < 0 || atoi(argv[5]) < 0 ) {
		cerr << "Error: Given region specs (x0, y0, width, height) must be non-negative." << endl;
		return -1;
	}

	// Attempt to instantiate and initialize a KernelObjTracker object
	KernelObjTracker kot( img, cvRect( atoi(argv[2]), atoi(argv[3]),
									   atoi(argv[4]), atoi(argv[5]) ),
						  16,      //number of histogram bins
						  1,       //bandwidth
						  false ); //adaptive bandwidth scaling?
	cvReleaseImage( &img ); // Init target image is no longer needed.

	// Open a window for visual feedback and enter main (tracking) loop
	cvNamedWindow( "track results", 1 );
	//cvNamedWindow( "prep", 1 );
	dispImage = NULL;
	while (cvWaitKey(10) != 27) {

		// Pull image from capture device
		if (!cvGrabFrame( capture ))
			break;
		img = cvRetrieveFrame( capture );

		// Kernel-based object tracking
		cout << "distance: " << kot.Track( img ) << endl;
		targetRegion = kot.GetTargetRegion();

		// Draw rectangles on image to show found "target candidate"
		if (dispImage)
			cvReleaseImage( &dispImage );
		dispImage = cvCloneImage( img );
		cvRectangle( dispImage,
					 cvPoint(targetRegion.x, targetRegion.y),
					 cvPoint(targetRegion.x+targetRegion.width, targetRegion.y+targetRegion.height),
					 CV_RGB( 255, 0, 0 ), 5 );
		cvShowImage( "track results", dispImage );

	}

	cvDestroyWindow( "track results" );
	//cvDestroyWindow( "prep" );
	cvReleaseCapture( &capture );

	return 0;
}
